import os
from click.testing import CliRunner
import pytest
import signal
import app.configfile as configfile
from app.commands import ls, add, move


def test_add(tmp_path):
    runner = CliRunner()
    configfile.CONFIG_FILE_PATH = './conf.yml'

    with runner.isolated_filesystem():
        result = runner.invoke(
            add, ['-d', './test/to', '-p', '.py', '-n', 'pyfiles'])
        assert result.exit_code == 0
        assert 'Error: Destination is not a valid directory\n' == result.output

        result = runner.invoke(
            add, ['-d', './test/to', '-p', '.py', '-n'])
        assert result.exit_code != 0  # Click error: missing arg
        result = runner.invoke(
            add, ['-d', './test/to', '-p', '.py'])
        assert result.exit_code != 0  # Click error: missing option

    with runner.isolated_filesystem():
        os.makedirs('./test/to')
        result = runner.invoke(
            add, ['-d', './test/to', '-p', '.py', '-n', 'pyfiles'])
        assert result.exit_code == 0
        config = configfile.Config({
            'name': 'pyfiles',
            'dest_dir': os.path.abspath('./test/from'),
            'pattern': '.py',
            'patterns': None
        })
        assert 'Added: Name=pyfiles' in result.stdout

        result = runner.invoke(
            add, ['-d', './test/to', '-p', '.py', '-n', 'pyfiles'])
        assert result.exit_code == 0
        assert result.stdout == 'Error: Configuration pyfiles already exists\n'


def test_ls():
    runner = CliRunner()
    configfile.CONFIG_FILE_PATH = './conf.yml'

    with runner.isolated_filesystem():
        os.makedirs('./test/to')
        result = runner.invoke(ls)
        assert result.exit_code == 0
        assert result.stdout == 'No configurations available\n'
        result = runner.invoke(
            add, ['-d', './test/to', '-p', '.py', '-n', 'pyfiles'])
        assert result.exit_code == 0
        result = runner.invoke(
            add, ['-d', './test/to', '-p', '.png', '-n', 'images'])
        assert result.exit_code == 0
        result = runner.invoke(ls)
        assert result.exit_code == 0
        assert 'Name=pyfiles' in result.stdout
        assert 'Name=images' in result.stdout


def test_move():
    runner = CliRunner()
    configfile.CONFIG_FILE_PATH = './conf.yml'

    with runner.isolated_filesystem():
        os.makedirs('./test/from')
        os.makedirs('./test/to')
        with open('./test/from/hello.py', 'w') as f:
            f.write("print('hello!'")
        result = runner.invoke(
            add, ['-d', './test/to', '-p', '.py', '-n', 'pyfiles'])
        assert result.exit_code == 0

        result = runner.invoke(
            move, ['-s', './test/from'])
        assert result.exit_code == 0
        assert 'hello.py' in result.stdout
        assert 'moved to' in result.stdout
        assert 'hello.py' in os.listdir('./test/to')
        assert 'hello.py' not in os.listdir('./test/from')

        # TODO add test for move and watch (-w option)



import os
import pytest
import app.configfile as configfile

SAMPLE_CONFIG = '''config:
- dest_dir: /Users/user/home/test/dest
  name: images
  pattern: .*
  patterns: []
- dest_dir: /Users/user/home/test/dest
  name: pyfiles
  pattern: .py$
  patterns: []'''


def _create_tmp_configfile(tmp_path, content=None):
    d = tmp_path / 'mmf-data-test'
    d.mkdir()
    f = d / 'conf.yml'
    content = content or SAMPLE_CONFIG
    f.write_text(content)
    assert f.read_text() == content
    configfile.CONFIG_FILE_PATH = f.as_posix()


def test_initialize_datadir(tmp_path):
    conf_file_path = tmp_path / 'mmf-data-test/conf.yml'
    configfile.CONFIG_FILE_PATH = conf_file_path.as_posix()
    configfile._initialize_datadir()
    assert os.path.exists(conf_file_path)


def test_load_configs_invalid(tmp_path):
    _create_tmp_configfile(tmp_path, 'invalid')
    with pytest.raises(configfile.Error):
        configs = configfile.load_configs()
        assert len(configs) == 0


def test_load_configs(tmp_path):
    # empty config file
    conf_file_path = tmp_path / 'mmf-data-test/conf.yml'
    configfile.CONFIG_FILE_PATH = conf_file_path.as_posix()
    configs = configfile.load_configs()
    assert len(configs) == 0

    # add new and test
    tmp_dir = tmp_path / 'target-dir'
    tmp_dir.mkdir()
    configfile.add_new_config(
        'new-config', tmp_dir.as_posix(), None, ['.png', '.jpg'])
    configs = configfile.load_configs()
    assert len(configs) == 1
    new_config = configs[0]
    assert new_config.name == 'new-config' and new_config.dest_dir == tmp_dir.as_posix()
    assert new_config.pattern == None and new_config.patterns == [
        '.png', '.jpg']


def test_add_new_config(tmp_path):
    conf_file_path = tmp_path / 'mmf-data-test/conf.yml'
    configfile.CONFIG_FILE_PATH = conf_file_path.as_posix()
    tmp_dir = tmp_path / 'target-dir'
    tmp_dir.mkdir()

    # error cases
    with pytest.raises(configfile.Error):
        configfile.add_new_config(
            'new-config', tmp_dir.as_posix(), None, None)
    with pytest.raises(configfile.Error):
        configfile.add_new_config(
            'new-config', '/invalid/dest/dir', '.py', None)
    with pytest.raises(configfile.Error):
        configfile.add_new_config(
            'new-config', tmp_dir.as_posix(), None, [])

    # non-error case
    configfile.add_new_config(
        'new-config', tmp_dir.as_posix(), None, ['.png', '.jpg'])
    configs = configfile.load_configs()
    assert len(configs) == 1
    new_config = configs[-1]
    assert new_config.name == 'new-config' and new_config.dest_dir == tmp_dir.as_posix()
    assert new_config.pattern == None and new_config.patterns == [
        '.png', '.jpg']



import click
from app.commands import move, add, ls


@click.group(help='Move My Files CLI tool')
def mmf():
    pass


mmf.add_command(move, 'move')
mmf.add_command(add, 'add')
mmf.add_command(ls, 'list')



import yaml
import os

# Configuration file path under HOME directory
CONFIG_FILE_PATH = os.path.expanduser('~') + '/mmf-data/conf.yml'


class Error(Exception):
    def __init__(self, message):
        self.message = message or 'default error message'

    def __str__(self):
        return f'Error: {self.message}'


class Config(yaml.YAMLObject):
    def __init__(self, obj):
        try:
            self.name = obj['name']
            self.dest_dir = obj['dest_dir']
            self.pattern = obj['pattern']
            self.patterns = obj['patterns']
        except:
            raise Error('Invalid configuration')

    def to_dict(self):
        return self.__dict__

    def __str__(self):
        patterns_str = ','.join(list(set(self.patterns or [self.pattern])))
        return f'Name={self.name}, ' + \
            f'Destination={self.dest_dir}, ' + \
            f'Patterns={patterns_str}'  # TODO improve pattern printing


def _initialize_datadir():
    if not os.path.exists(os.path.dirname(CONFIG_FILE_PATH)):
        try:
            os.makedirs(os.path.dirname(CONFIG_FILE_PATH))
        except:
            raise Error('Failed to initialize configuration directory')
    with open(CONFIG_FILE_PATH, 'w') as f:
        f.write('config: []')  # empty config


def load_configs():
    if not os.path.exists(CONFIG_FILE_PATH):
        _initialize_datadir()

    try:
        with open(CONFIG_FILE_PATH, 'r') as f:
            conf = f.read()
            yml = yaml.safe_load(conf)
            entries = yml['config']
            return [Config(e) for e in entries]
    except:
        raise Error('Failed to load configuration from mmf-data/conf.yml')


def add_new_config(name, dest_dir, pattern=None, patterns=None):
    if not os.path.exists(CONFIG_FILE_PATH):
        _initialize_datadir()

    if pattern is None and (patterns is None or len(patterns) == 0):
        raise Error('No pattern supplied')
    if not os.path.isdir(dest_dir):
        raise Error('Destination is not a valid directory')

    dest_dir = os.path.abspath(dest_dir)
    configs = load_configs()
    for c in configs:
        if c.name == name:
            raise Error(f'Configuration {name} already exists')

    configs.append(Config({
        'name': name,
        'dest_dir': dest_dir,
        'pattern': pattern,
        'patterns': patterns,
    }))
    _dump_configs(configs)
    return configs[-1]


def _dump_configs(configs):
    yml = {'config': []}
    for c in configs:
        yml['config'].append(c.to_dict())
    with open(CONFIG_FILE_PATH, 'w') as f:
        yaml.dump(yml, f)



from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='move-my-files',
    version='0.2.3',
    description='CLI tools to organize files on your computer',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/jan25/move-my-files',
    author='Abhilash Gnan',
    author_email='abhilashgnan@gmail.com',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3 :: Only',
    ],
    packages=find_packages(where='.'),
    package_dir={'': '.'},
    python_requires='>=3.5, <4',
    install_requires=[
        'watchdog',
        'Click',
        'pyyaml',
    ],
    extras_require={
        'dev': [
            'pytest',
        ]
    },
    entry_points={
        'console_scripts': [
            'mmf=app.cli:mmf',
        ],
    },
)



import shutil
import os
import sys
import re
import time
import click
import traceback
from watchdog.observers import Observer
from watchdog.events import LoggingEventHandler
import app.configfile as configfile


class Error(Exception):
    def __init__(self, message):
        self.message = message or 'default error message'

    def __str__(self):
        return f'Error: {self.message}'


def match(pat, filename):
    return re.search(pat, filename) is not None


def validate_srcdir(srcdir):
    if not os.path.exists(srcdir):
        raise Error('source directory does not exist')
    if not os.path.isdir(srcdir):
        raise Error('source is not a valid directory')


def validate_destdir(destdir):
    if not os.path.exists(destdir):
        raise Error('destination directory does not exist')
    if not os.path.isdir(destdir):
        raise Error('destination is not a valid directory')


def move_files(srcdir, pat, destdir):
    validate_srcdir(srcdir)
    validate_destdir(destdir)

    with os.scandir(srcdir) as it:
        for entry in it:
            if entry.is_file() and match(pat, entry.name):
                final_dest = shutil.move(entry.path, destdir)
                print(f'{entry.path} moved to {final_dest}')


def handle_one_off_move(srcdir, pat=None, destdir=None, name=None):
    validate_srcdir(srcdir)

    if not destdir:
        configs = configfile.load_configs()
        for config in configs:
            if name:
                if name == config.name:
                    move_files(srcdir, config.pattern, config.dest_dir)
            else:
                move_files(srcdir, config.pattern, config.dest_dir)
    else:
        move_files(srcdir, pat, destdir)


def handle_watch(srcdir, pat=None, destdir=None, name=None):
    validate_srcdir(srcdir)

    event_handler = LoggingEventHandler()
    observer = Observer()
    observer.schedule(event_handler, srcdir, recursive=False)
    observer.start()

    try:
        while True:
            handle_one_off_move(srcdir, pat, destdir, name)
            time.sleep(1)
    except KeyboardInterrupt:
        observer.stop()
    except Error as err:
        raise err
    finally:
        observer.join()


@click.command(help='Move files')
@click.option(
    '-s',
    '--source',
    'source',
    default='.',
    type=click.STRING,
    help='Source directory path. Defaults to current directory')
@click.option(
    '-d',
    '--dest-dir',
    type=click.STRING,
    help='Destination directory path')
@click.option(
    '-p',
    '--pattern',
    default='.*',
    type=click.STRING,
    help='Regex to match file names in source directory. Defaults to .* (all files)')
@click.option(
    '-n',
    '--name',
    type=click.STRING,
    help='Name of configuration to use')
@click.option(
    '-w',
    '--watch',
    is_flag=True,
    help='Watch a source directory for files to move')
def move(source, dest_dir, pattern, name, watch):
    try:
        if watch:
            handle_watch(source, pattern, dest_dir, name)
        else:
            handle_one_off_move(source, pattern, dest_dir, name)
    except Error as err:
        print(err)
    except:
        print(Error("Unexpected error"))
        traceback.print_exc()


@click.command(help='Add new configuration')
@click.option(
    '-d',
    '--dest-dir',
    required=True,
    type=click.STRING,
    help='Destination directory path')
@click.option(
    '-p',
    '--pattern',
    default='.*',
    type=click.STRING,
    help='Regex to match file names in source directory. Defaults to .* (all files)')
@click.option(
    '-n',
    '--name',
    required=True,
    type=click.STRING,
    help='Unique name for configuration')
def add(dest_dir, pattern, name):
    patterns = None  # TODO add support for multiple patterns
    try:
        config = configfile.add_new_config(name, dest_dir, pattern, patterns)
        click.echo(f'Added: {config}')
    except configfile.Error as err:
        click.echo(err)
    except:
        click.echo(Error("Unexpected error"))
        traceback.print_exc()


@click.command(help='List configurations')
def ls():
    try:
        configs = configfile.load_configs()
        if len(configs) > 0:
            for config in configs:
                click.echo(config)
        else:
            click.echo('No configurations available')
    except configfile.Error as err:
        click.echo(err)
    except:
        click.echo(Error("Unexpected error"))
        traceback.print_exc()
